package jain.application.services.jcp;

/*
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  Copyrights:
 *
 *  Copyright - 2001 Sun Microsystems, Inc. All rights reserved.
 *  901 San Antonio Road, Palo Alto, California 94043, U.S.A.
 *
 *  This product and related documentation are protected by copyright and
 *  distributed under licenses restricting its use, copying, distribution, and
 *  decompilation. No part of this product or related documentation may be
 *  reproduced in any form by any means without prior written authorization of
 *  Sun and its licensors, if any.
 *
 *  RESTRICTED RIGHTS LEGEND: Use, duplication, or disclosure by the United
 *  States Government is subject to the restrictions set forth in DFARS
 *  252.227-7013 (c)(1)(ii) and FAR 52.227-19.
 *
 *  The product described in this manual may be protected by one or more U.S.
 *  patents, foreign patents, or pending applications.
 *
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  Author:
 *
 *  Telcordia Technologies, Inc.
 *  Farooq Anjum, John-Luc Bakker, Ravi Jain
 *  445 South Street
 *  Morristown, NJ 07960
 *
 *  Version       : 1.0
 *  Notes         :
 *
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**
 A JcpCall is a transient association of (zero or more) addresses for the 
 purposes of engaging in a real-time communications interchange. The call
 and its associated connection and address objects describe the control
 and media flows taking place in some underlying "real world" communication 
 network. Other parties involved in the call may also exert control over it,
 thus the membership and state of the endpoints may change without 
 explicit request by the Jcp application. The {@link JcpProvider} adjusts the call, 
 address and connection objects to reflect the results of these 
 combined command actions. 
 * <H4>Introduction</H4>
 * A JcpCall can have zero or more {@link JcpConnection}s.
 * A two-party call has two JcpConnections, and a conference call
 * has three or  more JcpConnections. Each JcpConnection models the relationship
 * between a JcpCall and an JcpAddress, where an JcpAddress identifies a particular
 * party or set of parties on a call.
 *
 * <H4>Creating JcpCall Objects</H4>
 *
 * Applications create instances of a JcpCall object with the 
 * {@link JcpProvider#createCall()} method, which returns a JcpCall object that
 * has zero Connections and is in the {@link #IDLE} state.  The JcpCall
 * maintains a  reference to its JcpProvider for the life of that JcpCall object.
 * The JcpProvider object instance does not change throughout the lifetime of
 * the JcpCall object. The JcpProvider associated with a JcpCall is obtained via the
 * {@link #getProvider()} method.
 *
 * <H4>JcpCall States</H4>
 *
 * A JcpCall has a <EM>state</EM> which is obtained via the
 * {@link #getState()} method. This state describes the current
 * progress of a telephone call, where is it in its life cycle, and how many
 * connections exist on the call. The JcpCall state may be one of three values:
 * {@link #IDLE}, {@link #ACTIVE}, or
 * {@link #INVALID}. The following is a description of each state:
 *
 * <TABLE>
 * <TR>
 * <TD WIDTH="20%">{@link #IDLE}</TD>
 * <TD WIDTH="80%">
 * This is the initial state for all calls. In this state, the JcpCall has zero
 * connections, that is {@link #getConnections()} <EM>must</EM>
 * return null.
 * </TD>
 * </TR>
 *
 * <TR>
 * <TD WIDTH="20%">{@link #ACTIVE}</TD>
 * <TD WIDTH="80%">
 * A call with some current ongoing activity is in this state. JcpCalls with one
 * or more associated JcpConnections must be in this state. If a JcpCall is in this
 * state, the <CODE>getConnections()</CODE> method must return an array
 * of size at least one.
 * </TD>
 * </TR>
 *
 * <TR>
 * <TD WIDTH="20%">{@link #INVALID}</TD>
 * <TD WIDTH="80%">
 * This is the final state for all calls. JcpCall objects which lose all of their
 * JcpConnection objects (via a transition of the JcpConnection object into the
 * {@link JcpConnection#DISCONNECTED} state) moves into this state. Calls
 * in this state have zero JcpConnections and these JcpCall objects may not be used
 * for any future action. In this state, the
 * <CODE>getConnections()</CODE>
 * <EM>must</EM> return null.
 * </TD>
 * </TR>
 * </TABLE>
 *
 * <H4>JcpCall State Transitions</H4>
 *
 * Consider the transitions given below as the ones that would be visible to the application 
 * developer in the package considered. This may be either because there is a method in that 
 * class/interface that provides that behavior (read transition) or because of events in the 
 * underlying platform. Note that we are not making any statements to the effect that the other 
 * transitions (ones not shown) are disallowed. The application developer would have to keep 
 * in mind the possibility that the "invisible" transitions can surface (become visible) in 
 * inherited classes/interfaces.
 *
 * <CENTER>
 * <H1>JcpCall FSM</H1>
 * <IMG SRC="doc-files/core-callstates.gif">
 * </CENTER>
 * <H4>JcpCall and JcpConnection objects</H4>
 *
 * A JcpCall maintains a list of the JcpConnections on that JcpCall. Applications obtain
 * an array of JcpConnections associated with the JcpCall via the
 * <CODE>getConnections()</CODE> method. A JcpCall retains a reference to a
 * JcpConnection only if it is not in the {@link JcpConnection#DISCONNECTED}
 * state. Therefore, if a JcpCall has a reference to a JcpConnection, then that
 * JcpConnection must not be in the <CODE>JcpConnection.DISCONNECTED</CODE> state.
 * When a JcpConnection moves into the <CODE>JcpConnection.DISCONNECTED</CODE> state
 * (e.g. when a party hangs up), the JcpCall loses its reference to that
 * JcpConnection which is no longer reported via the
 * <CODE>JcpCall.getConnections()</CODE> method.
 *

 * 
 * <H4>Listeners and Events</H4>
 *
 * The {@link JcpCallListener} or {@link JcpConnectionListener} interface reports all events pertaining to
 * the JcpCall object. Events delivered to this interface must implement or extend the
 * {@link JcpCallEvent} interface. Applications can add listeners to a JcpCall
 * object
 * via the {@link #addCallListener(JcpCallListener)} method.
 * <p>
 * Connection-related events can be reported
 * via the 
 <CODE>JcpConnectionListener</CODE> interface. 
  These events include the
 * creation of these objects and their state changes. Events which are
 * reported
 * via the <CODE>JcpConnectionListener</CODE> interface pertaining to JcpConnections or JcpCalls
 *  extend the {@link JcpConnectionEvent} interface.
 *
 
       <H5><A NAME="event_snapshots">Event Snapshots</A></H5> 
       By default, when an listener is added 
       to a telephone call, the first batch of events may be a "snapshot".
       That is, if the listener was added after state changes in the Call,
       the first batch of events will inform the application of the 
       current state of the Call. Note that these snapshot events do NOT 
       provide a history of all events on the Call, rather they provide 
       the minimum necessary information to bring the application 
       up-to-date with the current state of the Call. 
 *
 
 * <H5>When Event Transmission Ends</H5>
 *
 * At times it may become impossible for the implementation to
 * report events to an application.  In this case,
 * a {@link JcpCallEvent#CALL_EVENT_TRANSMISSION_ENDED} is delivered to
 * an object registered as a <CODE>JcpCallListener</CODE>
 * (or an extension of that interface).
 * This is the final event received by the Listener.
*/
public interface JcpCall
{
    /**
        Retrieves the state of the call. The state will be either {@link #IDLE}, 
        {@link #ACTIVE} or {@link #INVALID}. 
        @return Integer representing the state of the call.
     */
    public int getState();

    /**
       Add a listener to this call. This also reports all state changes in the state of 
       the Call and Connection objects. The listener added with this method will 
       report events on the call for as long as the implementation can listen to 
       the Call. In the case that 
       <OL>
       <LI>the implementation can no longer observe the Call
       <LI>this  listener has been removed from this Call
       <LI>the Call has completed and moved to the {@link #INVALID} state 
       the application receives a {@link JcpCallEvent#CALL_EVENT_TRANSMISSION_ENDED} event. 
       </OL>
       
       <H5>CallListeners from Provider</H5> 
       There may be additional call listeners on the call which were not
       added by this method. These listeners may have become part of the 
       call via the {@link jain.application.services.jcc.JccProvider#addCallListener(JcpCallListener)} 
       or similar methods.

       <H5>ConnectionListeners</H5> 
        Since JcpConnectionListener inherits from the JcpCallListener, it is also 
        possible to add a JcpConnectionListener using this method. In such a case, 
        connection events  would also have to be reported to the
        registered listener in addition to the call events. 
        Hence, it is expected that the JCC implementation uses instanceof checks 
        in order to decide if only call events or both call and connection events 
        have to be delivered to the listener. Note that a listener added by this 
        method is expecting all the events without any filtering involved. 

       <H5>Multiple Invocations</H5> 
       If an application attempts to add an instance of a listener 
       already present on this Call, there are two possible outcomes:
       <OL>
        <LI>If the listener was added by the application using this 
        method, then a repeated invocation will silently fail, i.e. 
        multiple instances of an listener are not added and no exception
        will be thrown. 
        <LI>If the listener is part of the call because 
        an application invoked <CODE>JcpProvider.addCallListener()</CODE>
        either of these methods 
        modifies the behavior of that listener as if it were added via 
        this method instead. 
       </OL>

       <P><B>Post-Conditions:</B>
       <OL>
        <LI>A snapshot of events is delivered to the listener, if appropriate. 
       </OL>
       
       @param calllistener JcpCallListener object that receives the specified events.
        @throws MethodNotSupportedException The listener cannot be added at this time.
        @throws ResourceUnavailableException The resource limit for the number of 
        listeners has been exceeded. 
     */

    public void addCallListener( JcpCallListener calllistener)     throws ResourceUnavailableException, 
    MethodNotSupportedException;


    /**
        Removes a listener from this call. If successful, the listener will receive a 
        {@link JcpCallEvent#CALL_EVENT_TRANSMISSION_ENDED} as the last event it receives. If the listener is
        not part of the Call for the given address(es), then this method fails silently, i.e. no listener is removed
        and no exception is thrown.
        Note that the listeners will stop receiving events for existing calls also. 

         <p> This method has different effects depending upon how the 
         listener was added to the Call, as follows:
         <OL> <LI>If the listener was added via 
         {@link JcpCall#addCallListener(JcpCallListener)}, this method removes the 
         listener until it is re-applied by the application. 
         <LI>If the listener was added via 
         {@link jain.application.services.jcc.JccProvider#addCallListener(JcpCallListener)} 
         or simular methods, this method removes
         the listener for this call only. It does not affect whether this 
         listener will be added to future calls coming in to the JccProvider.
         </OL><p>

         <B>Post-Conditions:</B> <OL> 
         <LI>JcpCallEvent#CALL_EVENT_TRANSMISSION_ENDED is delivered to the 
         application </OL>

        @param calllistener JcpCall Listener object.
     */
    public void removeCallListener( JcpCallListener calllistener) ;


    /**
        Retrieves the provider handling this call object. The Provider reference does not
        change once the Call object has been created, despite the state of the 
        Call object. 
        
        @return JcpProvider object managing this call.
    */
    public JcpProvider getProvider();

    /**
        Retrieves an array of  connections asssociated with this call. None of the 
        Connections returned will be in the {@link JcpConnection#DISCONNECTED} state. Further, 
        if the Call is in the {@link #IDLE} or {@link #INVALID} state, 
        this method returns null.
        
        <p><B>Post-Conditions:</B> <OL> 
         <LI>JcpConnection[] conn = getConnections()<br>
         <LI>if this.getState() == IDLE then conn=null <br>
         <LI>if this.getState() == INVALID then conn=null <br>
         <LI>if this.getState() == ACTIVE then conn.length >=1 <br>
         <LI>For all i, conn[i].getState() != DISCONNECTED<br>
        </OL>

        @return Array of Connections for this call.
    */
    public JcpConnection[] getConnections();

    /**
        JcpCall.IDLE state indicates the Call has zero Connections. This is the initial 
        state of all Call objects.
    */
    public static final int IDLE      = 1;

    /**
        JcpCall.ACTIVE state indicates the Call has one or more Connections
        none of which is in the {@link JcpConnection#DISCONNECTED} state.  The 
        Call object transitions into this state from the {@link #IDLE} state only.  
    */
    public static final int ACTIVE    = 2;


    /**
        The JcpCall.INVALID state indicates that the Call has lost all of its connections, 
        that is, all of its Connection objects have moved into the {@link JcpConnection#DISCONNECTED}
        state and are no longer associated with the Call. 
         A Call in this state cannot
        be used for future actions.  
    */
    public static final int INVALID  = 3;
}
