package jain.application.services.jcp;

/*
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  Copyrights:
 *
 *  Copyright - 2001 Sun Microsystems, Inc. All rights reserved.
 *  901 San Antonio Road, Palo Alto, California 94043, U.S.A.
 *
 *  This product and related documentation are protected by copyright and
 *  distributed under licenses restricting its use, copying, distribution, and
 *  decompilation. No part of this product or related documentation may be
 *  reproduced in any form by any means without prior written authorization of
 *  Sun and its licensors, if any.
 *
 *  RESTRICTED RIGHTS LEGEND: Use, duplication, or disclosure by the United
 *  States Government is subject to the restrictions set forth in DFARS
 *  252.227-7013 (c)(1)(ii) and FAR 52.227-19.
 *
 *  The product described in this manual may be protected by one or more U.S.
 *  patents, foreign patents, or pending applications.
 *
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 *  Author:
 *
 *  Telcordia Technologies, Inc.
 *  Farooq Anjum, John-Luc Bakker, Ravi Jain
 *  445 South Street
 *  Morristown, NJ 07960
 *
 *  Version       : 1.0
 *  Notes         :
 *
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**
 * The <CODE>JcpPeerFactory</CODE> class is a class by which applications
 * obtain a JcpProvider object.
 * 
 * <H4> Introduction </H4>
 *
 * Applications use this class to first obtain a 
 * class which implements the {@link JcpPeer} interface. The
 * <CODE>JcpPeer</CODE> interface represents a particular vendor's
 * implementation of JCP.  The term 'peer' is Java nomenclature for
 * "a particular platform-specific implementation of a Java interface or API".
 * This term has the same meaning for the JAIN Core Package API. Applications are
 * not permitted to create an instance of the <CODE>JcpPeerFactory</CODE>
 * class. Through an installation procedure provided by each implementator,
 * a <CODE>JcpPeer</CODE> class is made available to an application
 * environment.  When applications have a JcpPeer object for a particular 
 * platform-dependent implementation, they may obtain a {@link JcpProvider} object via 
 * that interface. The details of that interface are discussed in the 
 * specification for the <CODE>JcpPeer</CODE> interface.
 *
 * <H4>Obtaining a JcpPeer Object</H4>
 *
 * Applications use the {@link #getJcpPeer(String)} method to
 * obtain a JcpPeer object. The argument to this method is a classname
 * which represents an object which implements the <CODE>JcpPeer</CODE>
 * interface. This object and the classname under which it can be found must
 * be supplied by the vendor of the implementation. Note that this object is
 * not a JcpProvider, however, this interface is used to obtain JcpProvider objects
 * from that particular implementation.
 * <p>
 * The JCP places conventions on vendors on the classname
 * they use for their JcpPeer object. This class name <EM>must</EM> begin
 * with the domain name assigned to the vendor in reverse order. Because the
 * space of domain names is managed, this scheme ensures that collisions
 * between two different vendor's implementations will not happen. For example,
 * an implementation from Sun Microsystem's will have "com.sun" as the prefix
 * to its JcpPeer class. After the reversed domain name, vendors are free to
 * choose any class hierarchy they desire.
 *
 * <H4>Default JcpPeer</H4>
 *
 * Additionally, the vendor providing the JcpPeer class may supply a
 * a <CODE>DefaultJcpPeer.class</CODE> class file. When placed in the 
 * classpath of applications, this class (which must implement the
 * <CODE>JcpPeer</CODE> interface) becomes the default JcpPeer object
 * returned by the {@link #getJcpPeer(String)} method. By 
 * convention the default class name must be <CODE>DefaultJcpPeer</CODE>.
 * <p>
 * In basic environments, applications and users do not want the burden of
 * finding out the class name in order to use a particular implementation.
 * Therefore, the <CODE>JcpPeerFactory</CODE> class supports a mechanism for
 * applications to obtain the default implementation for their system. If
 * applications use a <CODE>null</CODE> argument to the
 * {@link #getJcpPeer(String)} method, they will be returned
 * the default installed implementation on their system if it exists.
 * <p>
 * <STRONG>Note:</STRONG> It is the responsibility of implementation vendors
 * to supply a version of a <CODE>DefaultJcpPeer</CODE> or some means to
 * alias their peer implementation along with a means to place that
 * <CODE>DefaultJcpPeer</CODE> class in the application classpath.
 * <p>
 */

public class JcpPeerFactory {

  /**
   * Constructor for JcpPeerFactory class. This is private because
   * applications are not permitted to create an instance of the
   * JcpPeerFactory.
   */
  private JcpPeerFactory() {
  }


  /**
   * Returns an instance of a JcpPeer object given a fully qualified
   * classname of the class which implements the JcpPeer object.
   * <p>
   * If no classname is provided (null), a default class named 
   * <CODE>DefaultJcpPeer</CODE> is chosen as the classname to load.
   * If it does not exist or is not installed in the CLASSPATH as
   * the default, a ClassNotFoundException exception is thrown.
   * <p>
   
   * @param jcpPeerName The classname of the JcpPeer object class.
   * @return An instance of the JcpPeer object.
   * @exception ClassNotFoundException Indicates that the JcpPeer
   * specified by the classname is not available.
   */
  public synchronized static JcpPeer getJcpPeer(String jcpPeerName)
    throws ClassNotFoundException {
      
      // If jcpPeerName is null set it to a default jcpPeer name
      if (jcpPeerName == null || jcpPeerName.length() == 0) {
	jcpPeerName = getDefaultJcpPeerName();
      }

      // If the jcpPeerName is *still* null, then throw an exception
      if (jcpPeerName == null) {
	throw new ClassNotFoundException();
      }

      try {
	  Class jcpPeerClass = Class.forName(jcpPeerName);
	  
	  // Instantiate an instance of this class and cast it to the JcpPeer
	  // interface. If the class does not support the JcpPeer interface,
	  // an exception will be thrown because of the case, in addition to
	  // the exceptions thrown from the newInstance() method.
	  return (JcpPeer)jcpPeerClass.newInstance();
      }
      catch (Exception e) {
	String errmsg = "JcpPeer: " + jcpPeerName +
	  " could not be instantiated.";
	throw new ClassNotFoundException(errmsg);
      }
  }


  /**
   * Returns a default JcpPeer name. This method relies on the
   * existence of a default DefaultJcpPeer.class in the application
   * classpath.  Note that the name DefaultJcpPeer is a convention
   * enforced by this method.  Installations can alter this convention
   * by re-implementing this method.  This practice is in general
   * discouraged.
   * 
   * @return the default Jcp peer name
   */
  private static String getDefaultJcpPeerName() {
    String JcpPeerName = "DefaultJcpPeer";
    return JcpPeerName;
  }
}
